import javax.swing.JFrame;

/*
 * Klasse, welche fuer die Berechnung des SierpinskiDreiecks zustaendig ist
 */

public class SierpinskiTriangle {
	private Boolean[][] area; //Matrix aus boolschen Werten (Zelle an oder aus)
	private static int median; //Mittelwert der Feldgrosse, ab welchem das berechnen ausgefuehrt wird
	private static int[] ruleBinary; //Liste welche die Regeln zur berechnung der naechsten Generation enthaelt
	private int generations; //Anzahl der Generationen
	private int size; //Groesse des Zeichengeldes

	//Initialisierende Methode fuer die SierpinskiTriangle Klasse
	public SierpinskiTriangle(int rule, int size, int generations) {
		initializeSierpinskiArea(size, generations); //Initialisiert das Zeichenfeld mit der Groesse "size" und fuer "generations" generationen
		setupRules(rule); //Berechnen der Teilregeln fuer die Zahl in rule
	}
	
	//Aufrufmethode, mit welcher sich eine andere Klasse das berechnete Feld zurueckgeben lassen kann
	public Boolean[][] getArea(){
		return area; //Gibt den Inhalt in area zurueck
	}
	
	//Zeichenmethode fuer das SierpinskiDreieck 
	public void drawSierpinski() {
		area[median][0] = Boolean.TRUE; //Zuerst wird die erste Zelle auf wahr (an/schwarz) gesetzt
		Boolean prev, curr, next; //Variablen fuer die Vorgaengergeneration von Zellen
		for (int generation = 0; generation < generations-1; generation++) { //Eine Schleife, welche ueber die Anzahl der vorgegebenen Generationen traversiert
			for (int cell = 0; cell < size; cell++) { //Eine Schleife fuer jede Zelle in dieser Generation
				prev = getPrev(generation, cell); //Lade den Zustand der Zelle des vorherigen linken Nachbarn
				curr = getCurr(generation, cell); //Lade den Zustand der Zelle der aktuellen Zelle in der vorherigen Generation
				next = getNext(generation, cell); //Lade den Zustand der Zelle des vorherigen rechten Nachbarn
				area[cell][generation+1] = drawNextGenerationCell(prev,curr,next); //Zeichne den aus den drei vorherigen Zustaenden abgeleiteten Zustand in die aktuelle Zelle 
			}
		}
		
	}

	//Hauptzeichenmethode fuer die Matrix
	private Boolean drawNextGenerationCell(Boolean prev, Boolean curr, Boolean next) {
		Boolean output = Boolean.FALSE; //Variable fuer die Ausgabe des berrechneten Ergebnisses
		/*
		 * Der nachfolgende Abschnitt repraesentiert die Abfrage der drei Zustaende:
		 * 1. Zuerst wird geschaut ob der vorherige linke Nachbar schwarz oder weiss war
		 * 2. Dann wird geschaut ob die Zelle selbst in der vorherigen Generation schwarz oder weiss war
		 * 3. Zuletzt wird geschaut ob der vorherige rechte Nachbar schwarz oder weiss war
		 * Da sich die Nachfolgeschritte aus den acht Stellen der Binaerdarstellung der Regel ergeben werden 
		 * diese aus der vorher berechneten Regel liste ausgelesen. Jede Stelle in der Liste representiert
		 * eine Regel fuer die Zelle danach
		 */
		if (prev == Boolean.TRUE){
			if (curr == Boolean.TRUE){
				if (next == Boolean.TRUE){
					output = getBool(ruleBinary[0]);
				} else
				if (next == Boolean.FALSE){
					output = getBool(ruleBinary[1]);
				}
			} else
			if (curr == Boolean.FALSE){
				if (next == Boolean.TRUE){
					output = getBool(ruleBinary[2]);
				} else
				if (next == Boolean.FALSE){
					output = getBool(ruleBinary[3]);
				}
			}
		} else
		if (prev == Boolean.FALSE) {
			if (curr == Boolean.TRUE){
				if (next == Boolean.TRUE){
					output = getBool(ruleBinary[4]);
				} else
				if (next == Boolean.FALSE){
					output = getBool(ruleBinary[5]);
				}
			} else
			if (curr == Boolean.FALSE){
				if (next == Boolean.TRUE){
					output = getBool(ruleBinary[6]);
				} else
				if (next == Boolean.FALSE){
					output = getBool(ruleBinary[7]);
				}
			}
		} 
		return output; //Am Ende wird die Ausgabe zurueck gegeben
	}

	//Dies ist eine Hilfsfunktion, welche die Stellen der Binaerdarstellung der Regel als Boolsche Werte (schwarz/weiss) zurueck gibt
	private Boolean getBool(int i) {
		if (i == 1) { //wenn 1 dann
			return Boolean.TRUE; //True = Schwarz
		}
		else { //ansonsten
			return Boolean.FALSE; //False = Weiss
		}
	}

	//Hilfsfunktion um den vorherigen rechten Nachbarn der Zelle auszulesen
	private Boolean getNext(int generation, int cell) {
		if (cell+1 >= size) { //Abfrage ob das Ende des Rasters erreicht ist 
			return Boolean.FALSE; //Falls ja wird die vorherige rechte Nachbarzelle als weiss gehandhabt
		} else {
			return area[cell+1][generation]; //Ansonsten wird der eigentliche rechte vorherige Nachbar zurueck gegeben
		}
	}

	//Hilfsfunktion um den vorherigen Zustand der aktuellen Zelle auszulesen
	private Boolean getCurr(int generation, int cell) {
		return area[cell][generation]; //Gibt den Zustand des vorherigen Zustands der aktuellen Zelle zurueck
	}

	//Hilfsfunktion um den vorherigen linke Nachbarn der Zelle auszulesen
	private Boolean getPrev(int generation, int cell) { 
		if (cell-1 < 0) { //Abfrage ob das Ende des Rasters erreicht ist 
			return Boolean.FALSE; //Falls ja wird die vorherige linke Nachbarzelle als weiss gehandhabt
		} else {
			return area[cell-1][generation]; //Ansonsten wird der eigentliche linke vorherige Nachbar zurueck gegeben
		}
	}

	//Hilfsfunktion, welche die Zahl "rule" in dessen Binaerdarstellung umrechnet
	private static void setupRules(int rule) {
		ruleBinary = new int[8]; //Erstellen einer 8 Stelligen Liste
		int value = rule; //Uebergeben des Werts aus rule zu value
		int pos = 7; //Erste Position der Liste, welche beschrieben wird
		/*
		 * Um eine Dezimalzahl ins Binaersystem umzurechnen teilt man diese so lange durch 2, bis der Rest dieser Division 0 ergibt
		 * Den Rest der Division durch 2 erhaelt man mit modulo 2 (In Java ist es das % Zeichen). 
		 */
		while (pos > -1) { //Die Schleife wird so lange wiederholt bis alle 8 Stellen belegt wurden
			ruleBinary[pos] = value%2; //Belege die Stelle pos mit dem modulo 2 von value
			value = value/2; //Belege value mit dem ohne Rest durch 2 dividierten value
			pos--; //Zaehle die im naechsten Schritt zu befuellende Zelle runter
		}
		/*
		 * Diese vereinfachte Version der Umrechnung funktioniert nur fuer Zahlen von 0 bis 255, allerdings wird dieser Wert in diesem Programm
		 * nie erreicht, da die auswaehlbaren Regeln auf 8 beschraenkt wurde und keine groesser als 255 ist
		*/
	}

	//Initialisieren der Berechnungsflaeche des Fraktals
	private void initializeSierpinskiArea(int s, int g) {
		size = s; //Groesse des Fraktals wird mit s belegt
		generations = g; //Anzahl der Generationen wird mit g belegt
		area = new Boolean[size][generations+1]; //Neue Berechnungsflaeche wird erstellt
		median = size/2; //Mittelwert fuer die erste schwarze Zelle wird erstellt
		for (int i = 0; i < size; i++) { 
			area[i][0] = Boolean.FALSE; //Setze alle Werte der ersten Generation auf weiss
		}
		area[median][0] = Boolean.TRUE; //Setze den Wert der Zelle an der Stelle median auf schwarz
		/*
		 * Nach diesen Schritten wurde die erste Generation erstellt
		 */
	}

}